from torch.utils.data.dataset import T_co

from models.base.base_dataset import BaseDataset

import glob
import json
import math
import os
import random
import cv2

import numpy as np
import numpy.linalg as LA
import torch
from skimage import io
from torch.utils.data import Dataset
from torch.utils.data.dataloader import default_collate

import matplotlib.pyplot as plt


class WireDataset(BaseDataset):
    def __init__(self, images_dir, labels_dir, transform=None):
        # super().__init__()
        self.images_dir = images_dir
        self.labels_dir = labels_dir
        self.transform = transform
        # 假设文件名带jpg  这是所有的图片名
        self.image_files = [os.path.join(images_dir, f) for f in os.listdir(images_dir) if f.endswith('.jpg')]

        # print(f"n{images_dir}:", len(images_dir))
        # self.filelist = images_dir

    def __getitem__(self, idx) -> T_co:
        image_file = self.image_files[idx]
        image = cv2.cvtColor(cv2.imread(image_file), cv2.COLOR_BGR2RGB)
        label_file = os.path.join(self.labels_dir, os.path.basename(image_file).replace('.jpg', '.json'))

        target = self.read_target(item=idx, lbl_path=label_file)

        with open(label_file, 'r') as file:
            lable_all = json.load(file)

        n_stc_posl = 300
        n_stc_negl = 40
        use_cood = 0
        use_slop = 0

        wire = lable_all["wires"][0]  # 字典
        lpos = np.random.permutation(wire["line_pos_coords"]["content"])[: n_stc_posl]  # 不足，有多少取多少
        lneg = np.random.permutation(wire["line_neg_coords"]["content"])[: n_stc_negl]
        npos, nneg = len(lpos), len(lneg)
        lpre = np.concatenate([lpos, lneg], 0)  # 正负样本坐标合在一起
        for i in range(len(lpre)):
            if random.random() > 0.5:
                lpre[i] = lpre[i, ::-1]
        ldir = lpre[:, 0, :2] - lpre[:, 1, :2]
        ldir /= np.clip(LA.norm(ldir, axis=1, keepdims=True), 1e-6, None)
        feat = [
            lpre[:, :, :2].reshape(-1, 4) / 128 * use_cood,
            ldir * use_slop,
            lpre[:, :, 2],
        ]
        feat = np.concatenate(feat, 1)
        # print(torch.tensor(wire["junc_coords"]["content"])[:, :2])

        # meta = {
        #     "junc_coords": torch.from_numpy(wire["junc_coords"]["content"][:, :2]),
        #     "jtyp": torch.from_numpy(wire["junc_coords"]["content"][:, 2]).byte(),
        #     "line_pos_idx": self.adjacency_matrix(len(wire["junc_coords"]["content"]), wire["line_pos_idx"]["content"]),
        #     # 真实存在线条的邻接矩阵
        #     "line_neg_idx": self.adjacency_matrix(len(wire["junc_coords"]["content"]), wire["line_neg_idx"]["content"]),
        #     # 不存在线条的临界矩阵
        #     "lpre": torch.from_numpy(lpre[:, :, :2]),
        #     "lpre_label": torch.cat([torch.ones(npos), torch.zeros(nneg)]),  # 样本对应标签 1，0
        #     "lpre_feat": torch.from_numpy(feat),
        # }
        meta = {
            "junc_coords": torch.tensor(wire["junc_coords"]["content"])[:, :2],
            "jtyp": torch.tensor(wire["junc_coords"]["content"])[:, 2].byte(),
            "line_pos_idx": self.adjacency_matrix(len(wire["junc_coords"]["content"]), wire["line_pos_idx"]["content"]),
            # 真实存在线条的邻接矩阵
            "line_neg_idx": self.adjacency_matrix(len(wire["junc_coords"]["content"]), wire["line_neg_idx"]["content"]),
            # 不存在线条的临界矩阵
            "lpre": torch.tensor(lpre)[:, :, :2],
            "lpre_label": torch.cat([torch.ones(npos), torch.zeros(nneg)]),  # 样本对应标签 1，0
            "lpre_feat": torch.from_numpy(feat),
        }

        return torch.tensor(image).float(), meta, target
        # return super().__getitem__(index)

    def adjacency_matrix(self, n, link):   # 邻接矩阵
        mat = torch.zeros(n + 1, n + 1, dtype=torch.uint8)
        link = torch.tensor(link)
        if len(link) > 0:
            mat[link[:, 0], link[:, 1]] = 1
            mat[link[:, 1], link[:, 0]] = 1
        return mat

    def read_target(self, item, lbl_path, extra=None):
        # item同idx，lbl_path是lable路径
        # print(f'lbl_path:{lbl_path}')
        target = {}
        target["boxes"] = []
        target["segmentations"] = []
        with open(lbl_path, 'r') as file:
            lable_all = json.load(file)

        for i in lable_all["boxes"]:
            target["boxes"].append(i["data"])
        for i in lable_all["segmentations"]:
            target["segmentations"].append(i["data"])

        # target = {
        #     name: torch.tensor(target[name])
        #     for name in ["boxes", "segmentations"]
        # }
        return target

    def show(self, idx):  # 初步没问题
        image_file = self.image_files[idx]

        label_file = os.path.join(self.labels_dir, os.path.basename(image_file).replace('.jpg', '.json'))

        with open(label_file, 'r') as file:
            lable_all = json.load(file)

        # 可视化图像和标注
        image = cv2.imread(image_file)  # [H,W,3]  # 默认为BGR格式
        # 绘制每个标注的多边形
        for ann in lable_all["segmentations"]:
            segmentation = [ann['data']]
            if isinstance(segmentation, list):
                for seg in segmentation:
                    poly = np.array(seg).reshape((-1, 2)).astype(int)
                    cv2.polylines(image, [poly], isClosed=True, color=(0, 255, 0), thickness=2)
                    cv2.fillPoly(image, [poly], color=(0, 255, 0))

        # 显示图像
        cv2.namedWindow('Image with Segmentations', cv2.WINDOW_NORMAL)
        cv2.imshow('Image with Segmentations', image)
        cv2.waitKey(0)  # 等待按键事件，0表示无限等待
        cv2.destroyAllWindows()  # 关闭所有窗口


