import PIL
import cv2
import numpy as np
import torch
import torchvision
from matplotlib import pyplot as plt
from torchvision.io import read_image
from torchvision.models.detection import MaskRCNN_ResNet50_FPN_V2_Weights
from torchvision.models.detection.faster_rcnn import FastRCNNPredictor
from torchvision.models.detection.mask_rcnn import MaskRCNNPredictor
from torchvision.utils import draw_bounding_boxes

device = (
    "cuda"
    if torch.cuda.is_available()
    else "mps"
    if torch.backends.mps.is_available()
    else "cpu"
)

def show_img_boxes_masks(img, boxes, masks):
    show_img=img.cpu().permute(1, 2, 0).detach()
    masks=masks.cpu()
    boxes=boxes.cpu().detach()
    print(f'img:{img*255}')

    fig, axs = plt.subplots(3, 1,figsize=(30,20))

    # print(f'test_img:{test_img*255}')
    # axs[0].imshow(img)
    axs[0].imshow(show_img)

    axs[0].axis('off')
    axs[1].axis('off')
    axs[2].axis('off')
    drawn_boxes = draw_bounding_boxes((img*255).to(torch.uint8), boxes, colors="red", width=5)
    print(f'drawn_boxes:{drawn_boxes.shape}')
    # plt.imshow(drawn_boxes)
    axs[1].imshow(drawn_boxes.permute(1, 2, 0))

    img=img.cpu().permute(1, 2, 0).detach()


    for img_mask in masks:
        img_mask=img_mask.permute(1, 2, 0).detach()
        # img_mask=img_mask.unsqueeze(0)
        # img_mask = img_mask.expand_as(test_img)
        # print(f'img_mask:{img_mask.shape}')
        axs[2].imshow(show_img)
        axs[2].imshow(img_mask,alpha=0.3)

        # img_mask=np.array(img_mask)
        # print(f'img_mask:{img_mask.shape}')
        # plt.imshow(img_mask,alpha=0.5)
        # mask_3channel = cv2.merge([np.zeros_like(img_mask), np.zeros_like(img_mask), img_mask])
        # masked_image = cv2.addWeighted(test_img, 1, mask_3channel, 0.6, 0)

    # cv2.imshow('cv2 mask img', masked_image)
    # cv2.waitKey(0)

    plt.show()

def generate_colors(n):
    """
    生成n个均匀分布在HSV色彩空间中的颜色，并转换成BGR色彩空间。

    :param n: 需要的颜色数量
    :return: 一个包含n个颜色的列表，每个颜色为BGR格式的元组
    """
    hsv_colors = [(i / n * 180,1/3 * 255,2/3 * 255) for i in range(n)]
    bgr_colors = [tuple(map(int, cv2.cvtColor(np.uint8([[hsv]]), cv2.COLOR_HSV2BGR)[0][0])) for hsv in hsv_colors]
    return bgr_colors
def overlay_masks_on_image(image, masks, alpha=0.2):
    """
    在原图上叠加多个掩码，每个掩码使用不同的颜色。

    :param image: 原图 (NumPy 数组)
    :param masks: 掩码列表 (每个都是 NumPy 数组，二值图像)
    :param colors: 颜色列表 (每个颜色都是 (B, G, R) 格式的元组)
    :param alpha: 掩码的透明度 (0.0 到 1.0)
    :return: 叠加了多个掩码的图像
    """
    colors=generate_colors(len(masks))
    if len(masks) != len(colors):
        raise ValueError("The number of masks and colors must be the same.")

    # 复制原图，避免修改原始图像
    overlay = image.copy()

    for mask, color in zip(masks, colors):
        # 确保掩码是二值图像
        mask=mask.cpu().detach().permute(1,2,0).numpy()
        binary_mask = (mask > 0).astype(np.uint8) * 255  # 你可以根据实际情况调整阈值

        # 创建彩色掩码
        colored_mask = np.zeros_like(image)
        colored_mask[:] = color
        colored_mask = cv2.bitwise_and(colored_mask, colored_mask, mask=binary_mask)

        # 将彩色掩码与当前的叠加图像混合
        overlay = cv2.addWeighted(overlay,1-alpha, colored_mask, alpha, 0)

    return overlay
if __name__ == '__main__':
    model = torchvision.models.detection.maskrcnn_resnet50_fpn_v2(weights=MaskRCNN_ResNet50_FPN_V2_Weights.DEFAULT)
    in_features = model.roi_heads.box_predictor.cls_score.in_features

    model.roi_heads.box_predictor = FastRCNNPredictor(in_features, num_classes=5)
    in_features_mask = model.roi_heads.mask_predictor.conv5_mask.in_channels
    hidden_layer = 256
    model.roi_heads.mask_predictor = MaskRCNNPredictor(in_features_mask, hidden_layer, num_classes=5)
    transforms = MaskRCNN_ResNet50_FPN_V2_Weights.DEFAULT.transforms()
    print(f'transforms:{transforms}')
    model.to(device)

    state_dict = torch.load(r"F:\Downloads\maskrcnn_wts\best.pt")
    model.load_state_dict(state_dict)
    model.eval()
    # img_path=r"F:\DevTools\datasets\9\maskrcnn\images\train\2024-06-14-02-03-20_SaveImage.png"
    # img_path=r"D:\00030043_1.png"
    img_path=r"F:\DevTools\datasets\renyaun\1012\images\2024-09-23-09-59-03_SaveImage.png"
    img = read_image(img_path)
    img=transforms(img)
    img=img.to(device)
    # img = torchvision.transforms.ToTensor()(img)
    print(f'img shape:{img.shape}')
    result=model([img])
    print(f'result:{result}')
    masks=result[0]['masks']
    boxes=result[0]['boxes']
    # cv2.imshow('mask',masks[0].cpu().detach().numpy())
    boxes = boxes.cpu().detach()
    drawn_boxes = draw_bounding_boxes((img * 255).to(torch.uint8), boxes, colors="red", width=5)
    print(f'drawn_boxes:{drawn_boxes.shape}')
    boxed_img=drawn_boxes.permute(1,2,0).numpy()


    mask=masks[0].cpu().detach().permute(1,2,0).numpy()

    mask=cv2.resize(mask,(800,800))
    # cv2.imshow('mask',mask)
    img=img.cpu().detach().permute(1,2,0).numpy()
    # cv2.imshow('img',img)
    masked_img=overlay_masks_on_image(boxed_img,masks)
    masked_img=cv2.resize(masked_img,(800,800))
    cv2.imshow('img_masks',masked_img)

    boxed_img = cv2.resize(boxed_img, (800, 800))
    cv2.imshow('boxes', boxed_img)
    # show_img_boxes_masks(img, boxes, masks)
    cv2.waitKey(0)
